import java.util.zip.Inflater;
import java.io.*;
import java.util.ArrayList;
import java.security.MessageDigest;
import java.util.zip.*;
import java.util.Iterator;
/*
 * Created on Jan 15, 2008
 */

/**
 * @author Bobby
 */
public class JJ2LevelStats {

	public static void main(String[] args) throws IOException {
		JJ2LevelStats jls;
		if (args.length < 1)
			jls = new JJ2LevelStats();
		else
			jls = new JJ2LevelStats(args[0]);
			
		// Will print info on all of the j2l files in the given directory (does not recurse).
		//jls.inFolder = "C:/games/tsf";
		//printNoZip(jls);
		
		// Will recurse directory below and print all of the j2l files from all of the zip files below 
		ZipFileRecurse zfr = new ZipFileRecurse(new File("D:/bob/jazz2online/public_html/J2Ov2"));
		ArrayList al = zfr.getFiles();
		System.err.println(al.size() + " zip files found.");
		int i=1;
		Iterator it = al.iterator();
		while (it.hasNext()) {
			File f = (File)it.next();
			if (i % 100 == 0)
				System.err.println("Working on file " + i + " :" + f);
			JJ2Zip jz = new JJ2Zip(f);
			jz.printAllStats();
			i++;
		}		
	}	
	
	public String inFolder;
	
	public JJ2LevelStats() {
		inFolder = ".";
	}
	
	public JJ2LevelStats(String in)	{
		inFolder = in;
	}
	
	public File[] fileList() {
	    File dir = new File(inFolder);
	    // It is also possible to filter the list of returned files.
	    // This example does not return any files that start with `.'.
	    FilenameFilter filter = new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            if(name.toLowerCase().endsWith(".j2l")) {
	            	try {
	            		File f = new File(dir + "\\" + name);
	            		if (f.length() == 0) {
	            			System.err.println("Bad j2l file: " + name);
	            			return false;
	            		}
	            		byte[] b = Utilities.getBytesFromFile(f);
	            		
	            		byte[] b2 = new byte[J2LHeader.HEADER.length];
	            		System.arraycopy(b, 0, b2, 0, b2.length);
	            		
	            		boolean equals = true;
	            		for (int i=0; i<b2.length; i++)
	            		{
	            			if (b2[i] != J2LHeader.HEADER[i]) {
	            				equals = false;
	            				System.err.println("Bad j2l file: " + name);
	            				break;
	            			}
	            		}
	            		return equals;
	            	} catch (IOException e) {}
	            	return false;
	        	} else {
	        		return false;
	        	}
	            
	        }
	    };
	    return dir.listFiles(filter);		
	}
	
	public static void printNoZip(JJ2LevelStats jls) throws IOException
	{
		File[] files = jls.fileList();
		for(int i=0; i<files.length; i++)
		{
			LevelStats l = new LevelStats(files[i]);
			l.printStats();
		}
	}
}

class JJ2Zip {
	public File file;
	public JJ2Zip(File f) {
		file = f;
	}
	
	private boolean isLegalJ2LFile(byte[] b, String name) {
		boolean equals = true;
		for (int i=0; i<J2LHeader.HEADER.length; i++)
		{
			if (b[i] != J2LHeader.HEADER[i]) {
				equals = false;
				System.err.println("Bad j2l file: " + file.getPath() + ":" + name);
				break;
			}
		}
		return equals;
	}
	
	public void printAllStats() {
		try {
			FileInputStream fis = new FileInputStream(file);
			ZipInputStream zis = new ZipInputStream(fis);
			
			ZipEntry ze;
			while((ze=zis.getNextEntry())!=null){
				if (ze.getName().toLowerCase().endsWith(".j2l") && ze.getSize() > 0) {
					byte[] b = Utilities.loadBytesFromStreamForSize(zis, (int)ze.getSize());
					
					if (isLegalJ2LFile(b, ze.getName()))
					{
						// good file
						LevelStats l = new LevelStats(file.getPath(), b, ze);
						l.printStats();
					}
				}
				zis.closeEntry();
			}
			zis.close();
			
			
		} catch(FileNotFoundException e) { System.err.println(file + " not found."); }
		  catch(IOException e) { System.err.println("IO Exception - probably a corrupt zip file!");
		  						System.err.println("  " + file);
		  					   }
		  catch(IllegalArgumentException e) { System.err.println("Illegal Argument"); 
		  									  System.err.println("  " + file); }
	}
}

class Utilities {
    // Returns the contents of the file in a byte array.
    // FROM: http://www.exampledepot.com/egs/java.io/File2ByteArray.html
	public static byte[] getBytesFromFile(File file) throws IOException {
        InputStream is = new FileInputStream(file);
    
        // Get the size of the file
        long length = file.length();
    
        // You cannot create an array using a long type.
        // It needs to be an int type.
        // Before converting to an int type, check
        // to ensure that file is not larger than Integer.MAX_VALUE.
        if (length > Integer.MAX_VALUE) {
            // File is too large
        }
    
        // Create the byte array to hold the data
        byte[] bytes = new byte[(int)length];
    
        // Read in the bytes
        int offset = 0;
        int numRead = 0;
        while (offset < bytes.length
               && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
            offset += numRead;
        }
    
        // Ensure all the bytes have been read in
        if (offset < bytes.length) {
            throw new IOException("Could not completely read file "+file.getName());
        }
    
        // Close the input stream and return bytes
        is.close();
        return bytes;
    }
	
	// FROM: http://www.captain.at/howto-java-convert-binary-data.php	
	public static int arr2int (byte[] arr, int start) {
		int low = arr[start] & 0xff;
		int high = arr[start+1] & 0xff;
		return (int)( high << 8 | low );
	}	
	
	// FROM: http://www.captain.at/howto-java-convert-binary-data.php
	public static long arr2long (byte[] arr, int start) {
		int i = 0;
		int len = 4;
		int cnt = 0;
		byte[] tmp = new byte[len];
		for (i = start; i < (start + len); i++) {
			tmp[cnt] = arr[i];
			cnt++;
		}
		long accum = 0;
		i = 0;
		for ( int shiftBy = 0; shiftBy < 32; shiftBy += 8 ) {
			accum |= ( (long)( tmp[i] & 0xff ) ) << shiftBy;
			i++;
		}
		return accum;
	}
	
	  /**
	   * FROM: http://java.happycodings.com/Other/code2.html
	   * Loads the supplied number bytes from the given input stream
	   *
	   * @param stream to read the bytes from
	   * @param the number of bytes to read
	   * @return bytes read from the stream
	   * @exception java.io.IOException reports IO failures
	   */
	  public static byte[] loadBytesFromStreamForSize( InputStream in, int size )
	      throws IOException {
	    int count, index = 0;
	    byte[] b = new byte[size];

	    // read in the bytes from input stream
	    while( ( count = in.read( b, index, size ) ) > 0 ) {
	      size -= count;
	      index += count;
	    }
	    return b;
	  }
}

class J2LHeader
{
	public static final byte[] HEADER = {0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x4a,0x61,0x7a,0x7a,0x20,0x4a,0x61,0x63,0x6b,0x72,0x61,0x62,0x62,0x69,0x74,0x20,0x32,0x20,0x44,0x61,0x74,0x61,0x20,0x46,0x69,0x6c,0x65,0x0d,0x0a,0x0d,0x0a,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x52,0x65,0x74,0x61,0x69,0x6c,0x20,0x64,0x69,0x73,0x74,0x72,0x69,0x62,0x75,0x74,0x69,0x6f,0x6e,0x20,0x6f,0x66,0x20,0x74,0x68,0x69,0x73,0x20,0x64,0x61,0x74,0x61,0x20,0x69,0x73,0x20,0x70,0x72,0x6f,0x68,0x69,0x62,0x69,0x74,0x65,0x64,0x20,0x77,0x69,0x74,0x68,0x6f,0x75,0x74,0x0d,0x0a,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x77,0x72,0x69,0x74,0x74,0x65,0x6e,0x20,0x70,0x65,0x72,0x6d,0x69,0x73,0x73,0x69,0x6f,0x6e,0x20,0x66,0x72,0x6f,0x6d,0x20,0x45,0x70,0x69,0x63,0x20,0x4d,0x65,0x67,0x61,0x47,0x61,0x6d,0x65,0x73,0x2c,0x20,0x49,0x6e,0x63,0x2e,0x0d,0x0a,0x0d,0x0a,0x1a,0x4c,0x45,0x56,0x4c};
	public static final int LEVELNAME = 188;
	public static final int LEVELNAME_LENGTH = 32;
	public static final int VERSION = 220;
	public static final int CDATA1 = 230;
	public static final int CDATA1_LENGTH = 4;
	public static final int UDATA1 = 234;
	public static final int UDATA1_LENGTH = 4;
}

class J2LData1
{
	public static final int START = 262;
	
	public static final int LEVELNAME = 19;
	public static final int TILESET = 51;
	public static final int BONUS = 83;
	public static final int NEXT = 115;
	public static final int SECRET = 147;
	public static final int MUSIC = 179;
	public static final int STRLEN = 32;
	
	public static final int LAYER4_W = 8463;
	public static final int LAYER4_H = 8527;
}

class LevelStats {
	//public File filename;
	
	public String location;
	public String filename;
	public byte[] byteFile; 
	public long lastModified;
	
	public LevelStats(String loc, byte[] b, ZipEntry ze) {
		byteFile = b;
		filename = ze.getName();
		location = loc;
		lastModified = ze.getTime();
	}
	
	public LevelStats(File file) throws IOException {
		location = file.getPath();
		filename = file.getName();
		byteFile = Utilities.getBytesFromFile(file);
		lastModified = file.lastModified();
	}
	
	public String getLevelName(byte[] file) {
		return new String(file, J2LHeader.LEVELNAME, J2LHeader.LEVELNAME_LENGTH).trim();
	}

	public byte[] getData1(byte[] file)
	{
		long size = Utilities.arr2long(file, J2LHeader.UDATA1);
		byte[] output = new byte[(int)size];

		int cdata1Length = (int)Utilities.arr2long(file, J2LHeader.CDATA1);

		Inflater decompresser = new Inflater();
		decompresser.setInput(file, J2LData1.START, cdata1Length);
		try {
			decompresser.inflate(output);
		} catch(Exception e) {}
		decompresser.end();
		return output;
	}

	public String getTileset(byte[] data1) {
		return new String(data1, J2LData1.TILESET, J2LData1.STRLEN).trim();
	}
	
	public String getMusic(byte[] data1) {
		return new String(data1, J2LData1.MUSIC, J2LData1.STRLEN).trim();
	}	

	public String getNext(byte[] data1) {
		return new String(data1, J2LData1.NEXT, J2LData1.STRLEN).trim();
	}	
	
	public int getLayer4W(byte[] data1) {
		return (int)Utilities.arr2long(data1, J2LData1.LAYER4_W);
	}
	
	public int getLayer4H(byte[] data1) {
		return (int)Utilities.arr2long(data1, J2LData1.LAYER4_H);
	}	

	public int getVersion(byte[] file) {
		return Utilities.arr2int(file, J2LHeader.VERSION);
	}
	
	public String getMD5(byte[] file)
	{
		try {
			MessageDigest digest = java.security.MessageDigest.getInstance("MD5");
	    	digest.update(file);
	    	return HexString.bufferToHex(digest.digest());
		} catch(Exception e) { }
		return "!!!BADMD5!!";
	}
	
	public void printStats() throws IOException
	{
		byte[] data1 = getData1(byteFile);
		
		System.out.println(
				esc(location) + "," +
				esc(filename) + "," +
				getVersion(byteFile) + "," +
				byteFile.length + "," +
				lastModified + "," +
				getMD5(byteFile) + "," +
				esc(getLevelName(byteFile)) + "," +
				esc(getTileset(data1)) + "," +
				esc(getMusic(data1)) + "," +
				esc(getNext(data1)) + "," +
				getLayer4W(data1) + "," +
				getLayer4H(data1)
			);
	}
	
	private String esc(String s)
	{
		s = s.replaceAll("\\\"","\\\\\"");
		return "\"" + s + "\"";
	}
}

class ZipFileRecurse {
	public File startDir;
	public ZipFileRecurse(File startDir) {
		this.startDir = startDir;
	}
	
	public ArrayList h = new ArrayList();
	
	private void addDirContents(File dir) {
	    FilenameFilter filter = new FilenameFilter() {
	        public boolean accept(File dir, String name) {
	            return name.toLowerCase().endsWith(".zip") || new File(dir + "\\" + name).isDirectory();
	        }
	    };
	    
	    File[] files = dir.listFiles(filter);
	    for(int i=0; i<files.length; i++) {
	    	if (files[i].isDirectory())
	    		addDirContents(files[i]);
	    	else
	    		h.add(files[i]);
	    }
	}
	
	public ArrayList getFiles() {
		h.clear();
		addDirContents(startDir);
		return h;
	}
}